package com.mortals.xhx.busiz.web;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.xhx.busiz.req.*;
import com.mortals.xhx.busiz.rsp.AppealStatInfo;
import com.mortals.xhx.common.code.PerformTypeEnum;
import com.mortals.xhx.common.code.ProcessStatusEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.check.model.*;
import com.mortals.xhx.module.check.service.*;
import com.mortals.xhx.module.perform.model.*;
import com.mortals.xhx.module.perform.service.*;
import com.mortals.xhx.module.staff.model.StaffEntity;
import com.mortals.xhx.module.staff.model.StaffQuery;
import com.mortals.xhx.module.staff.service.StaffService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import static com.mortals.framework.ap.SysConstains.*;
import static com.mortals.xhx.common.key.ErrorCode.ERROR_TOKEN_EXPIRED;
import static com.mortals.xhx.common.key.ErrorCode.ERROR_TOKEN_EXPIRED_CONTENT;

/**
 * h5 巡查
 *
 * @author: zxfei
 * @date: 2023/7/7 15:08
 */
@RestController
@Slf4j
@RequestMapping("/api/v1/inspect")
public class InspectApiController extends AbstractBaseController<PerformReq> {




    @Autowired
    private PerformAttendRecordService attendRecordService;
    @Autowired
    private PerformReviewRecordService reviewRecordService;
    @Autowired
    private PerformComplainRecordService complainRecordService;
    @Autowired
    private PerformGoworkRecordService performGoworkRecordService;
    @Autowired
    private PerformEffectRecordService effectRecordService;
    @Autowired
    private PerformOtherRecordService otherRecordService;
    @Autowired
    private PerformRulesService rulesService;
    @Autowired
    private StaffService staffService;


    /**
     * 巡查新增
     */
    @PostMapping(value = "save")
    public Rest<PerformAttendAppealEntity> appealSave(@RequestBody InspectSaveReq req) {
        String busiDesc = "个人巡查记录新增";
        log.info("【{}】【请求体】--> {}", busiDesc, JSONObject.toJSONString(req));
        Rest<PerformAttendAppealEntity> rest = Rest.ok(busiDesc + " 【成功】");
        Context context = this.getContext();
        try {
            //根据状态 查询 当前是否已经存在正在进行的申诉
            if (ObjectUtils.isEmpty(req.getPerformType())) throw new AppException("绩效类型不能为空！");
            if (ObjectUtils.isEmpty(req.getPhone())) throw new AppException("手机号码不能为空！");
            if (ObjectUtils.isEmpty(req.getRuleCode())) throw new AppException("绩效规则编码不能为空！");


            if (PerformTypeEnum.考勤绩效.getValue().equals(req.getPerformType())) {
                boolean bool = attend(req);
                if (!bool) throw new AppException(busiDesc + "失败！");
            } else if (PerformTypeEnum.评价差评绩效.getValue().equals(req.getPerformType())) {
                boolean bool = review(req);
                if (!bool) throw new AppException(busiDesc + "失败！");
            } else if (PerformTypeEnum.评价投诉绩效.getValue().equals(req.getPerformType())) {
                boolean bool = complain(req);
                if (!bool) throw new AppException(busiDesc + "失败！");
            } else if (PerformTypeEnum.办件绩效.getValue().equals(req.getPerformType())) {
                boolean bool = gowork(req);
                if (!bool) throw new AppException(busiDesc + "失败！");
            } else if (PerformTypeEnum.效能绩效.getValue().equals(req.getPerformType())) {
                boolean bool = effect(req);
                if (!bool) throw new AppException(busiDesc + "失败！");
            } else if (PerformTypeEnum.其它绩效.getValue().equals(req.getPerformType())) {
                boolean bool = other(req);
                if (!bool) throw new AppException(busiDesc + "失败！");
            } else {
                throw new AppException("不支持当前绩效类型");
            }
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error(busiDesc, e);
            rest = Rest.fail(super.convertException(e));
        }
        log.info("【{}】【响应体】--> {}", busiDesc, JSONObject.toJSONString(req));
        return rest;
    }





    private boolean attend(InspectSaveReq req) throws AppException {
        //考勤保存
        //通过手机号码查询员工属性
        PerformRulesEntity rule = getRule(req.getRuleCode());
        StaffEntity staffEntity = getStaff(req.getPhone());

        PerformAttendRecordEntity recordEntity = new PerformAttendRecordEntity();
        recordEntity.initAttrValue();
        BeanUtils.copyProperties(req, recordEntity);
        recordEntity.setStaffId(staffEntity.getId());
        recordEntity.setStaffName(staffEntity.getName());


        recordEntity.setSubAddType(rule.getSubAddType());
        recordEntity.setScore(rule.getScore());
        recordEntity.setRuleId(rule.getId());
        recordEntity.setRuleNme(rule.getName());
        recordEntity.setCategoryId(rule.getCategoryId());
        recordEntity.setCategoryName(rule.getCategoryName());

        recordEntity.setCreateUserId(1L);
        recordEntity.setCreateTime(new Date());
        PerformAttendRecordEntity save = attendRecordService.save(recordEntity);

        return save.newEntity();

    }


    private boolean review(InspectSaveReq req) throws AppException {
        //评价保存
        StaffEntity staffEntity = getStaff(req.getPhone());
        PerformRulesEntity rule = getRule(req.getRuleCode());

        PerformReviewRecordEntity recordEntity = new PerformReviewRecordEntity();

        recordEntity.initAttrValue();
        BeanUtils.copyProperties(req, recordEntity);
        recordEntity.setStaffId(staffEntity.getId());
        recordEntity.setStaffName(staffEntity.getName());
        recordEntity.setSubAddType(rule.getSubAddType());
        recordEntity.setScore(rule.getScore());
        recordEntity.setRuleId(rule.getId());
        recordEntity.setRuleName(rule.getName());
        recordEntity.setCategoryId(rule.getCategoryId());
        recordEntity.setCategoryName(rule.getCategoryName());

        recordEntity.setCreateUserId(1L);
        recordEntity.setCreateTime(new Date());
        PerformReviewRecordEntity save = reviewRecordService.save(recordEntity);
        return save.newEntity();

    }

    private boolean complain(InspectSaveReq req) throws AppException {
        //投诉保存
        StaffEntity staffEntity = getStaff(req.getPhone());
        PerformRulesEntity rule = getRule(req.getRuleCode());

        PerformComplainRecordEntity recordEntity = new PerformComplainRecordEntity();

        recordEntity.initAttrValue();
        BeanUtils.copyProperties(req, recordEntity);
        recordEntity.setStaffId(staffEntity.getId());
        recordEntity.setStaffName(staffEntity.getName());
        recordEntity.setSubAddType(rule.getSubAddType());
        recordEntity.setScore(rule.getScore());
        recordEntity.setRuleId(rule.getId());
        recordEntity.setRuleName(rule.getName());
        recordEntity.setCategoryId(rule.getCategoryId());
        recordEntity.setCategoryName(rule.getCategoryName());

        recordEntity.setCreateUserId(1L);
        recordEntity.setCreateTime(new Date());
        PerformComplainRecordEntity save = complainRecordService.save(recordEntity);
        return save.newEntity();

    }

    private boolean gowork(InspectSaveReq req) throws AppException {
        //办件保存
        StaffEntity staffEntity = getStaff(req.getPhone());
        PerformRulesEntity rule = getRule(req.getRuleCode());

        PerformGoworkRecordEntity recordEntity = new PerformGoworkRecordEntity();

        recordEntity.initAttrValue();
        BeanUtils.copyProperties(req, recordEntity);
        recordEntity.setStaffId(staffEntity.getId());
        recordEntity.setStaffName(staffEntity.getName());
        recordEntity.setSubAddType(rule.getSubAddType());
        recordEntity.setScore(rule.getScore());
        recordEntity.setRuleId(rule.getId());
        recordEntity.setRuleName(rule.getName());
        recordEntity.setCategoryId(rule.getCategoryId());
        recordEntity.setCategoryName(rule.getCategoryName());

        recordEntity.setCreateUserId(1L);
        recordEntity.setCreateTime(new Date());
        PerformGoworkRecordEntity save = performGoworkRecordService.save(recordEntity);
        return save.newEntity();

    }

    private boolean effect(InspectSaveReq req) throws AppException {
        //效能保存
        StaffEntity staffEntity = getStaff(req.getPhone());
        PerformRulesEntity rule = getRule(req.getRuleCode());

        PerformEffectRecordEntity recordEntity = new PerformEffectRecordEntity();

        recordEntity.initAttrValue();
        BeanUtils.copyProperties(req, recordEntity);
        recordEntity.setStaffId(staffEntity.getId());
        recordEntity.setStaffName(staffEntity.getName());
        recordEntity.setSubAddType(rule.getSubAddType());
        recordEntity.setScore(rule.getScore());
        recordEntity.setRuleId(rule.getId());
        recordEntity.setRuleName(rule.getName());
        recordEntity.setCategoryId(rule.getCategoryId());
        recordEntity.setCategoryName(rule.getCategoryName());

        recordEntity.setCreateUserId(1L);
        recordEntity.setCreateTime(new Date());
        PerformEffectRecordEntity save = effectRecordService.save(recordEntity);
        return save.newEntity();
    }

    private boolean other(InspectSaveReq req) throws AppException {
        //其它保存

        StaffEntity staffEntity = getStaff(req.getPhone());
        PerformRulesEntity rule = getRule(req.getRuleCode());

        PerformOtherRecordEntity recordEntity = new PerformOtherRecordEntity();

        recordEntity.initAttrValue();
        BeanUtils.copyProperties(req, recordEntity);
        recordEntity.setStaffId(staffEntity.getId());
        recordEntity.setStaffName(staffEntity.getName());
        recordEntity.setSubAddType(rule.getSubAddType());
        recordEntity.setScore(rule.getScore());
        recordEntity.setRuleId(rule.getId());
        recordEntity.setRuleName(rule.getName());
        recordEntity.setCategoryId(rule.getCategoryId());
        recordEntity.setCategoryName(rule.getCategoryName());

        recordEntity.setCreateUserId(1L);
        recordEntity.setCreateTime(new Date());
        PerformOtherRecordEntity save = otherRecordService.save(recordEntity);
        return save.newEntity();
    }


    private PerformRulesEntity getRule(String ruleCode) {
        PerformRulesEntity rule = rulesService.getCache(ruleCode);
        if (ObjectUtils.isEmpty(rule))
            throw new AppException(String.format("当前手机号码未找到匹配的员工！rule:%s", ruleCode));

        return rule;
    }

    private StaffEntity getStaff(String phone) {
        StaffEntity staffEntity = staffService.selectOne(new StaffQuery().phoneNumber(phone));
        if (ObjectUtils.isEmpty(staffEntity))
            throw new AppException(String.format("当前手机号码未找到匹配的员工！phone:%s", phone));

        return staffEntity;
    }



    protected void parsePageInfo(Map<String, Object> model, PageInfo pageInfo) {
        model.put(TOTAL, pageInfo.getTotalResult());
        model.put(PER_PAGE, pageInfo.getPrePageResult());
        model.put(CURRENT_PAGE, pageInfo.getCurrPage());
        model.put(LAST_PAGE, pageInfo.getTotalPage());
        model.put(PAGEINFO_KEY, pageInfo);
    }


    public static void main(String[] args) {

    }

}
